;QuadCombo - Usese similar decoding at in ComboJos.asm 
;  It is a four outlet popwer strip controlled by
;  coded touch plate input to turn circuits on and off

; Wayne Simister wsimister@sisna.com

;Because this circuit goes to sleep, it can be well adapted to 1.5 volt (stepped up) battery operation.
;    Power could be drawn only when button is pushed or unit is being programmed.

; Five bytes reserved for switching code.  One or all can be used.
;EEPROM contains permanent code which must be programmed before use
;  (first byte contains number of bits programmed)

;Dit-Dah entries used for code to toggle AC circuits.

;HoldDown (HC) count of 15 allows entry of code for outlet 1
;   HC 16 to program code for outlet 2
;   HC 17 to program code for outlet 3
;   HC 18 to program code for outlet 4
;   Dit-Dah entries from one up to 255 are permitted for each code.

;  R18,0(1) = Relay One flag (1=on)  These flags indicate relay status of on or off
;  R18,1(2) = Relay Two flag (2=on)
;  R18,2(4) = Relay Three flag (3=on)
;  R18,3(8) = Relay Four flag (4=on)

;Current draw at idle =  

;Internal oscillator in ATTINY2313 is running micro-controller at 1MHz

;February 16, 2008

;----------------------------------------------------

;       HOLDDOWN, LED FLASH, COMMANDS:

; 15. Unit 1 Enter New Code:   Enter new code using dit-dah entries. One to 255 dit-dah entries possible.
;    (this is entered into EEPROM for permanent storage after a power down)
; 16 Enters code for Outlet 2
; 17 Enters code for Outlet 3
; 18 Enters code for Outlet 4


;r0-r4 = SRAM TP code entry registers (r0=No. of bits, r1-4 = 4 bytes for code)
;r5-r9 = SRAM duplicate code copied from EEPROM (r5= No. of bits, r6-9 = 4 bytes)
;r10-r12 = Long Delay countdown "escape" timer registers

; SRAM
; $60 = SREG save

; $65-$78 = 20 storage locations (5 each) for relays 1 through 4 combination code

;-------------------------------------------------------

.include "tn2313def.inc"

rjmp	START1
rjmp	Decode


START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set bits 0-3 for output
out		DDRA,r16	;Execute data direction with PA0-3 as output

clr		r16
out		PortA,r16	;turn off ports

ldi		R16,251		;Set bit 2(4) as input ... remaining bits as output
out		DDRD,R16	;execute data direction with bit 2 as input
out		PORTD,R16	;execute to set out ports high, and in ports with
					; no input resistor

in		r16,GIMSK	;enable INT0 (D port, bit 2(4))
ori		r16,64		; (bit 6 = INT0)
out		GIMSK,r16

in		r16,MCUCR	;set up MCUCR for power down and low level sensing
ori		r16,16		;  power down bit set to one = power down
andi		r16,252		;  low level sensing for INT0 (bits 0,1 to zero)
out		MCUCR,r16

ser		R16			;Set all bits for data direction as outputs
out		DDRB,R16	;execute data direction with all as outputs
clr		r16
out		PORTB,R16	;execute to set out ports low

clr		r18			;clear flags register R18

rcall	Delay60flash		;Delay to allow capacitor to fully charge

rcall	FetchEEPROM	;grab last recorded code data from EEPROM and put in SRAM

rcall	InitializeRelays	;cycle all relays on and off to be sure they are in correct operating state after bootup

sei			;enable global interrupts


;-----------------------------

PowerDown:		;Power down and loop to await interrupt

in		r16,MCUCR	;Set SE bit 5(32) for entering sleep mode
ori		r16,32
out		MCUCR,r16
sleep				;Go to sleep - power down mode

rjmp	PowerDown


;--------------------------------------

Decode:		;Interrupt Routine: Decodes touches from touch plate


push		r16		;save value in r16

in		r16,SREG
sts		$60,r16		;Save SREG contents in RAM

in		r16,MCUCR	;Clear SE bit 5(32) to prevent accidental sleep
andi	r16,223		
out		MCUCR,r16

rcall	Button

;Check for any hold down counts

cpi		r20,1		;off relay circuit 1 if equal
brne	CHC1
rcall	PulseOff1	;turn relay #1 off
rjmp	DecodeEnd	;exit


CHC1:

cpi		r20,2		;off relay circuit 2 if equal
brne	CHC2
rcall	PulseOff2		;off relay #2
rjmp	DecodeEnd


CHC2:

cpi		r20,3		;off relay circuit 3 if equal
brne	CHC3
rcall	PulseOff3		;off relay #3
rjmp	DecodeEnd


CHC3:

cpi		r20,4		;off relay circuit 4 if equal
brne	CHC4
rcall	PulseOff4		;off relay #4
rjmp	DecodeEnd


CHC4:

cpi		r20,5	
brne	CHC5
rcall	PulseOff1
rcall	PulseOff2
rcall	PulseOff3
rcall	PulseOff4
rjmp	DecodeEnd


CHC5:

cpi		r20,15		;compare to see if HC is 15 (code relay one)
brne	CHC6
rcall	LongFlash
rcall	ButtonStall	;time to enter code -- wait until button pushed
ldi		r30,$00		;set up EEPROM store address for first relay code (z register pair)
ldi		r28,$65		;set up SRAM store address for first relay code (y register pair)
rcall	WriteCode
rjmp	DecodeEnd


CHC6:

cpi		r20,16		;compare to see if HC is 16 (code relay two)
brne	CHC7
rcall	LongFlash
rcall	ButtonStall	;time to enter code -- wait until button pushed
ldi		r30,$05		;set up EEPROM store address for second relay code 
ldi		r28,$6A		;set up SRAM store address
rcall	WriteCode
rjmp	DecodeEnd


CHC7:

cpi		r20,17		;compare to see if HC is 17 (code relay three)
brne	CHC8
rcall	LongFlash
rcall	ButtonStall	;time to enter code -- wait until button pushed
ldi		r30,$0A		;set up EEPROM store address for third relay code
ldi		r28,$6F		;set up SRAM store address
rcall	WriteCode
rjmp	DecodeEnd


CHC8:

cpi		r20,18		;compare to see if HC is 18 (code relay four)
brne	CHC9
rcall	LongFlash
rcall	ButtonStall	;time to enter code -- wait until button pushed
ldi		r30,$0F		;set up EEPROM store address for fourth relay code
ldi		r28,$74		;set up SRAM store address for fourth relay code
rcall	WriteCode
rjmp	DecodeEnd


CHC9:

cpi		r20,20		;Put in RAM ONLY -- rly1=1 rly2=2 rly3=3 fly4=4
brne	CHC10
rcall	FlashTwo	
rcall	RamOnly1234
rjmp	DecodeEnd


CHC10:

cpi		r20,22		;Put in EEPROM and RAM -- rly1=1 rly2=2 rly3=3 rly4=4
brne	CHC11
rcall	FlashTwo
rcall	EEPROM1234
rjmp	DecodeEnd


CHC11:

cpi		r20,24		;re-install old code now in EEPROM
brne	CHC19
rcall	FlashTwo
rcall	FetchEEPROM
rjmp	DecodeEnd


;following will decode an entry of dit-dah coding

CHC19:

ldi		r30,$65		;set start of RAM for circuit #1 code
rcall	CheckMatch
cpi		r31,1		;is match (1) found?
brne	CHC20
rcall	Toggle1		;toggle relay #1


CHC20:

ldi		r30,$6A		;get start of RAM for circuit #2 code
rcall	CheckMatch
cpi		r31,1		;is match (1) found?
brne	CHC21
rcall	Toggle2		;toggle relay#2


CHC21:

ldi		r30,$6F		;get start of RAM for circuit #3 code
rcall	CheckMatch
cpi		r31,1		;is match (1) found?	
brne	CHC22
rcall	Toggle3		;toggle relay#3


CHC22:

ldi 	r30,$74		;get start of RAM for circuit #4 code
rcall	CheckMatch
cpi		r31,1		;is match (1) found?
brne	DecodeEnd
rcall	Toggle4		;toggle relay#4


DecodeEnd:

rcall	FlashFour

lds		r16,$60			;retrieve SREG
out		SREG,r16
pop		r16			;restore r16 value

reti				;return from interrupt


;------------------------

Delay40th:			; 1/40th second delay

push	r31
push	r0
push	r1

ldi		r31,33		; high byte


Delay40tha:

mov		r1,r31		; set up r1 for 1/40th second delay
clr		r0
ldi		r31,1		;no delay for r31


DelayA:

dec		r0
brne	DelayA

dec		r1
brne	DelayA

dec		r31
brne	DelayA

pop		r1
pop		r0
pop		r31

ret

;----------------------------------

DelayThreeFourths:	;3/4 second delay

push	r16
ldi		r16,30
rjmp	Delay6a


;----------------------------------


DelayOne:		;1 second delay

push	r16
ldi		r16,40
rjmp	Delay6a

;------------------------------------


Delay6:				;6 second delay

push	r16
clr		r16


Delay6a:

rcall	Delay40th
dec		r16
brne	Delay6a

pop		r16
ret


;---------------------------------------

Delay60flash:		;approximate 60 second delay routine with blink
					;  of LED  (Actual measured delay = 1:09)

push 	r0
push	r16
clr		r0
ldi		r16,3
mov		r0,r16
clr		r16


Delay45a:

sbi		PortD,6		;on LED
rcall	Delay40th	;1/40th second delay
cbi		PortD,6		;off LED
rcall	Delay40th
rcall	Delay40th
dec		r16
brne	Delay45a

dec		r0
brne	Delay45a

pop		r16
pop		r0
ret


;------------------------------------

FlashOnce:

push	r16
ldi		r16,1
rjmp	FF1


;---------------------------

FlashTwo:

push	r16
ldi		r16,2
rjmp	FF1


;-----------------------------

FlashFour:			;rapid four flashes of LED

push r16

ldi	r16,4


FF1:

rcall	Delay40th
sbi		PortD,6		;on LED
rcall	Delay40th
cbi		PortD,6		;off LED
rcall	Delay40th

dec		r16
brne	FF1

pop		r16
ret


;-----------------------------------

LongFlash:

sbi		PortD,6				;on LED
rcall	DelayThreeFourths
cbi		PortD,6				;off LED
ret


;-----------------------------------

VariableFlashBoth:

VF1:

sbi		PortD,6		;on PROGRAM LED
rcall	Delay40th
cbi		PortD,6		;off PROGRAM LED
rcall	Delay40th

dec		r16
brne	VF1

ret


;---------------------------------------

ButtonStall:

sbic	PIND,2		;skip if switch is down
rjmp	ButtonStall

;---------------------------------------

Button:		;Replaces old ButtonPush.  1/11/2006

push	r23

clr		r20		;HoldDown count
clr		r0		;clear number of bits in code - bit count
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)

clr		r23


B1:

inc		r23
rcall	Delay40th
cpi		r23,120		;delay before first holddown count - 40=one second - 3 seconds
brsh	B3			;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

sec					;set carry flag -- it is dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortD,6
rcall	Delay40th
rcall	Delay40th	;delay for longer on of LEDs
rcall	Delay40th
rcall	Delay40th


B2c:

rcall	Delay40th
cbi		PortD,6

inc		r0			;increment bit count
clr		r23			;clear r23


B2d:

sbis	PIND,2		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,120		;has switch been up 120 counts? 40=one second (3 seconds)
brsh	B5

rjmp	B2d


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

clc					;clear carry flag -- it is dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortD,6

rjmp	B2c


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi		PortD,6
rcall	Delay40th
cbi		PortD,6		;flash PROGRAM LED

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,2		;skip if switch is down
rjmp	B4a

cpi		r23,30
brlo	B4

rjmp	B3


B4a:

rcall	Delay40th


B5:

pop		r23
ret


;---------------------------------------

; January 30, 2008 written

FetchEEPROM:		;grabs data in permanent saved EEPROM and saves in SRAM
			;   - r16 has EEPROM address, r17 has data address, r30 has SRAM address
			;    (unused byte of Z register pair, r31 is used for scratch)

ldi		r30,$65		;start of SRAM store area (20 locations)
clr		r16			;start of EEPROM store area


FEE1:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE1

out		EEAR,r16		;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r17,EEDR		;read data from data register

st		z+,r17		;store data (r17) in SRAM with post increment of r30
inc		r16

cpi		r16,20		;see if all 20 bytes of data has been read.  If not, loop, otherwise halt
brne	FEE1

ret					

;-------------------------------------------------

; February 7, 2008 written

WriteEEPROM:		;write EEPROM byte - r30 (z register) has EEPROM address,
					; r16 has data, 
                                                
sbic	EECR,EEPE	;wait until ready before continuing
rjmp	WriteEEPROM

cli			;Off interrupts so write not corrupted if interrupt occurs

out		EEAR,r30		;set up address in address register
out		EEDR,r16		;put contents of r16 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

sei					;On interrupts 
ret	

; ----------------------------------------

WriteCode:		;r30 (z register pair) has EEPROM address, 
				;r28 (y register pair) has SRAM address

mov		r16,r0		;write first byte
rcall	WriteEEPROM
inc		r30			;increment z pointer (r30)
st		y+,r0		;store in SRAM	

mov		r16,r1		;write second byte
rcall	WriteEEPROM
inc		r30			;increment z pointer (r30)
st		y+,r1		;store in SRAM

mov		r16,r2		;write third byte
rcall	WriteEEPROM
inc		r30			;increment z pointer (r30)
st		y+,r2		;store in SRAM	

mov		r16,r3		;write fourth byte
rcall	WriteEEPROM
inc		r30			;increment z pointer (r30)
st		y+,r3		;store in SRAM

mov		r16,r4		;write fifth byte
rcall	WriteEEPROM
st		y+,r4		;store in SRAM

ret


;--------------------------------

CheckMatch:	;  z register (r30) contains RAM address for data transfer

clr		r31	; zero match flag r31=0

ld		r5,z+
ld		r6,z+
ld		r7,z+
ld		r8,z+
ld		r9,z	;load from RAM, code data for comparison

CM1:

or		r5,r5	;check for zero.  If zero there is no code ... EXIT
breq	CMEnd

cp		r0,r5
brne	CMEnd

cp		r1,r6
brne	CMEnd

cp		r2,r7
brne	CMEnd

cp		r3,r8
brne	CMEnd

cp		r4,r9
brne	CMEnd

CM2:			;Match has been reached

inc		r31		;Set r31=1 to indicate match

CMEnd:

ret


;---------------------------------------------------------

;  TOGGLE RELAYS SECTION

Toggle1:

sbrs	r18,0		;skip if bit 0,(1) is set
rjmp	TR1a


PulseOff1:

cbr		r18,1		;clear r18, bit 0(1) off flag for relay
sbi		PortB,1		;set bit 1(2) to pulse relay off
rcall	Delay40th
cbi		PortB,1		;return bit 1(2) to off
ret


TR1a:

sbr		r18,1		;set r18, bit 0(1) on flag for relay
sbi		PortB,0		;set bit 0(1) to pulse relay on
rcall	Delay40th	
cbi		PortB,0		;return bit 0(1) to off
ret


;------------------------------------------

Toggle2:

sbrs	r18,1		;skip if R18, bit 1,(2) is set
rjmp	TR2a


PulseOff2:

cbr		r18,2		;clear r18, bit 1(2) off flag for relay
sbi		PortB,3		;set bit 3(8) to pulse relay off
rcall	Delay40th
cbi		PortB,3		;return bit 3(8) to off
ret


TR2a:

sbr		r18,2		;set r18, bit 1(2) on flag for relay
sbi		PortB,2		;set bit 2(4) to pulse relay on
rcall	Delay40th	
cbi		PortB,2		;return bit 2(4) to off
ret


;---------------------------------------

Toggle3:

sbrs	r18,2		;skip if bit 2,(4) is set
rjmp	TR3a


PulseOff3:

cbr		r18,4		;clear r18, bit 2(4) off flag for relay
sbi		PortB,5		;set bit 5(32) to pulse relay off
rcall	Delay40th
cbi		PortB,5		;return bit 5(32) to off
ret


TR3a:

sbr		r18,4		;set r18, bit 2(4) on flag for relay
sbi		PortB,4		;set bit 4(16) to pulse relay on
rcall	Delay40th	
cbi		PortB,4		;return bit 4(16) to off
ret


;----------------------------------------

Toggle4:

sbrs	r18,3		;skip if bit 3,(8) is set
rjmp	TR4a


PulseOff4:

cbr		r18,8		;clear r18, bit 3(8) off flag for relay
sbi		PortB,7		;set bit 7(128) to pulse relay off
rcall	Delay40th
cbi		PortB,7		;return bit 7(128) to off
ret


TR4a:

sbr		r18,8		;set r18, bit 3(8) on flag for relay
sbi		PortB,6		;set bit 6(64) to pulse relay on
rcall	Delay40th	
cbi		PortB,6		;return bit 6(64) to off
ret


;--------------------------------------

InitializeRelays:		; cycles each relay on then off for initialization at bootup

ldi		r16,1
clr		r17				;zero in r17

IR1:

out		PortB,r16		; On or Off selected relay (even ports=0N, odd ports=OFF)
rcall	Delay40th		; pulse selected relay
out		PortB,r17		; Off relays
rcall	FlashOnce		; Flash LED

rcall	DelayOne	; one second delay before next iteration
lsl		r16			; move bit to next highest position 

brcc	IR1			; branch back if carry cleared

ret					; All relays now in off state after this routine


;-------------------------------

RamOnly1234:

clr		r0
clr		r1
clr		r2
clr		r3
clr		r4		;clear all registers
ldi		r30,$65	;set up z register (r30) for RAM install

ROa:

inc		r0			;start with one, then two, etc.
st		z+,r0		;load RAM with 1,2,3,4 touch code
st		z+,r1
st		z+,r2
st		z+,r3
st		z+,r4
mov		r31,r0
cpi		r31,4		;stop after 4th group has been installed
brne	ROa

ret


;-------------------------

EEPROM1234:

rcall	RamOnly1234	;load RAM with 1,2,3,4 touch code
ldi		r30,$00		;set start of EEPROM memory
ldi		r28,$65		;set start of RAM memory section


EEPROMa:

ld		r16,y+		;load RAM data into r16
					; (y reg (r28) points to RAM address)

rcall	WriteEEPROM	;write data into EEPROM
inc		r30			;increment z pointer (r30)

cpi		r28,$79		;if at end of RAM, then exit
brne	EEPROMa

ret	


;---------------------------




